"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const wt = require("waves-transactions");
const waves_crypto_1 = require("waves-crypto");
const axios_1 = require("axios");
const ride_js_1 = require("@waves/ride-js");
class WavesConsoleAPI {
    constructor() {
        this.file = (tabName) => (WavesConsoleAPI.env.editors.filter((e) => e.label == tabName)[0] || { code: '' }).code;
        this.contract = () => {
            try {
                return WavesConsoleAPI.env.editors[WavesConsoleAPI.env.selectedEditor].code;
            }
            catch (e) {
                throw new Error('No active contract tab found');
            }
        };
        this.keyPair = (seed) => waves_crypto_1.keyPair(seed || WavesConsoleAPI.env.SEED);
        this.publicKey = (seed) => this.keyPair(seed).public;
        this.privateKey = (seed) => this.keyPair(seed).private;
        this.address = (seed, chainId) => waves_crypto_1.address(seed || WavesConsoleAPI.env.SEED, chainId || WavesConsoleAPI.env.CHAIN_ID);
        this.compile = (code) => {
            const r = ride_js_1.compile(code);
            if (r.error)
                throw new Error(r.error);
            return this.bufferToBase64(new Uint8Array(r.result));
        };
        this.broadcast = (tx, apiBase) => __awaiter(this, void 0, void 0, function* () {
            const url = new URL('transactions/broadcast', apiBase || WavesConsoleAPI.env.API_BASE).href;
            try {
                const resp = yield axios_1.default.post(url, tx);
                return resp.data;
            }
            catch (e) {
                if (e.response && e.response.status === 400) {
                    throw new Error(e.response.data.message);
                }
                else
                    throw e;
            }
        });
        this.deploy = (params, seed) => __awaiter(this, void 0, void 0, function* () {
            let txParams = params || {};
            txParams.script = txParams.script === undefined ? this.compile(this.contract()) : txParams.script;
            const setScriptTx = this['setScript'](txParams, seed);
            return this.broadcast(setScriptTx);
        });
        this.help = (func) => {
            var pos = -1, al0 = '', type = typeof func, params = [], aliases = [];
            // Try to find function name
            for (al0 in this) {
                if ((type == 'undefined' || func == this[al0])) {
                    aliases.push(al0);
                }
            }
            // Sort functions list and move help help to the top
            if (aliases.length > 1) {
                aliases.sort((a, b) => {
                    if (a > b) {
                        return 1;
                    }
                    else if (a < b) {
                        return -1;
                    }
                    return 0;
                });
                // Get position of help in list
                pos = aliases.indexOf('help');
                // Move help to the top of list
                aliases.unshift(aliases.splice(pos, 1)[0]);
            }
            // Compile help text from pieces
            return WavesConsoleAPIHelp.compileText(aliases);
        };
        Object.keys(wt).forEach(key => {
            this[key] = (params, seed) => wt[key](seed || WavesConsoleAPI.env.SEED, Object.assign({ chainId: WavesConsoleAPI.env.CHAIN_ID }, params));
        });
    }
    static setEnv(env) {
        WavesConsoleAPI.env = env;
    }
    bufferToBase64(buf) {
        const binstr = Array.prototype.map.call(buf, (ch) => String.fromCharCode(ch)).join('');
        return btoa(binstr);
    }
}
exports.WavesConsoleAPI = WavesConsoleAPI;
/**
 * Help parsers and text compilers
 *
 * @static
 * @class WavesConsoleAPIHelp
 */
class WavesConsoleAPIHelp {
    /**
     * Generates API method argument(s) whole description
     *
     * @static
     * @method compileText
     *
     * @param {Array} aliases
     *
     * @returns {string}
     */
    static compileText(aliases) {
        var last = aliases.length - 1, module = WavesConsoleAPIHelp, full = aliases.length == 1 ? true : false, text = '';
        // Compile text for each command
        aliases.forEach((alias, index) => {
            text = this.compileTextSlice(alias, full, text);
            // Add ; or .
            if (!full) {
                if (index == last) {
                    text = `${text}.`;
                }
                else {
                    text = `${text};`;
                }
            }
        });
        // Add header for commands list
        if (full === false) {
            text = `${module.common.list.header}\n${text}`;
        }
        return text;
    }
    /**
     * Generates API method arguments description
     *
     * @static
     * @method compileTextArguments
     *
     * @param {Array} args
     * @param {string} text
     *
     * @returns {string}
     */
    static compileTextArguments(args, text) {
        var last = args.length - 1, type = '', summary = '', module = WavesConsoleAPIHelp;
        // Add arguments list header
        text = `${text}\n\n${module.common.args.header}`;
        // Add each argument info
        args.forEach((argument, index) => {
            text = `${text}\n${index + 1}. ${argument}`;
            if (module.types[argument]) {
                summary = module.types[argument].summary;
                summary = summary.substring(0, 1).toLowerCase() + summary.substring(1);
                type = module.types[argument].type;
                type = type ? type : '';
                // Add argument type
                if (type) {
                    text = `${text}: ${type}`;
                }
                // Add argument summary
                if (summary) {
                    text = `${text} — ${summary}`;
                }
                // Add ; or .
                if (index == last) {
                    text = `${text}.`;
                }
                else {
                    text = `${text};`;
                }
            }
        });
        return text;
    }
    /**
     * Generates API specified method description
     *
     * @static
     * @method compileTextSlice
     *
     * @param {string} alias
     * @param {boolean} full
     * @param {string} text
     *
     * @returns {string}
     */
    static compileTextSlice(alias, full, text) {
        var module = WavesConsoleAPIHelp, summary = '', params = module.texts[alias].params || [], description = '', type = '', vals, args = params.slice();
        // Check optional and obligatory function params
        if (args) {
            args = args.map((arg) => {
                return module.types[arg].optional ? `[${arg}]` : `${arg}`;
            });
        }
        // Add common function info
        if (full) {
            text = `${alias}(${args.join(', ')})`;
        }
        else {
            text = `${text}\n${alias}(${args.join(', ')})`;
        }
        if (module.texts[alias]) {
            // Add summary text
            if (module.texts[alias].summary) {
                summary = module.texts[alias].summary;
                summary = summary.substring(0, 1).toLowerCase() +
                    summary.substring(1);
                text = `${text} — ${summary}`;
                if (full) {
                    text = `${text}.`;
                }
            }
            // Add arguments description
            if (full && args.length) {
                text = module.compileTextArguments(params, text);
            }
            // Add full description
            if (full && module.texts[alias].description) {
                description = module.texts[alias].description;
                text = `${text}\n\n${description}`;
            }
        }
        return text;
    }
}
/**
 * Common texts pieces
 *
 * @static
 * @member {object} common
 */
WavesConsoleAPIHelp.common = {
    list: {
        header: 'Available functions:'
    },
    args: {
        header: 'Arguments:'
    }
};
/**
 * Commands descriptions vocabulary
 *
 * @static
 * @member {object} texts
 */
WavesConsoleAPIHelp.texts = {
    file: {
        summary: '' +
            'Gets editor contents for tab',
        description: '' +
            'Used inside web-ide or vscode plugin.',
        params: ['tabName']
    },
    data: {
        summary: '' +
            'Creates signed data transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed DataTransaction as a second agrument.',
        params: ['params', 'seed']
    },
    issue: {
        summary: '' +
            'Creates signed issue transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed IssueTransaction as a second agrument.',
        params: ['params', 'seed']
    },
    contract: {
        summary: '' +
            'Open editor tab content',
        description: '' +
            ''
    },
    keyPair: {
        summary: '' +
            'Generates keyPair from seed',
        description: '' +
            '',
        params: ['seed']
    },
    publicKey: {
        summary: '' +
            'Generates publicKey from seed',
        description: '' +
            '',
        params: ['seed']
    },
    privateKey: {
        summary: '' +
            'Generates privateKey from seed',
        description: '' +
            '',
        params: ['seed']
    },
    address: {
        summary: '' +
            'Generates address from seed',
        description: '' +
            '',
        params: ['seed']
    },
    compile: {
        summary: '' +
            'Gets editor contents for tab',
        description: '' +
            'Accepts plain text of a contract as an argument. Returns compiled contract in base64.',
        params: ['code']
    },
    broadcast: {
        summary: '' +
            'Sends transaction to the Waves network using REST API',
        description: '' +
            'Returns Promise.',
        params: ['tx', 'apiBase']
    },
    deploy: {
        summary: '' +
            'Compile currently selected contract and deploy it to default account',
        description: '' +
            '',
        params: ['params', 'seed']
    },
    help: {
        summary: '' +
            'Help for the available API functions',
        description: '' +
            'You can use help() to get list of available functions ' +
            'or help(functionName) to get info for the specified function.',
        params: ['func']
    },
    transfer: {
        summary: '' +
            'Creates signed transfer transaction',
        description: '' +
            '',
        params: ['params', 'seed']
    },
    massTransfer: {
        summary: '' +
            'Creates signed massTransfer transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed MassTransferTransaction as a second agrument.',
        params: ['params', 'seed']
    },
    reissue: {
        summary: '' +
            'Creates signed reissue transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use already ' +
            'signed ReissueTransaction as a second agrument.',
        params: ['params', 'seed']
    },
    burn: {
        summary: '' +
            'Creates signed burn transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed BurnTransaction as a second agrument.',
        params: ['params', 'seed']
    },
    lease: {
        summary: '' +
            'Creates signed lease transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed LeaseTransaction as a second agrument.'
    },
    cancelLease: {
        summary: '' +
            'Creates signed cancelLease transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed CancelLeaseTransaction as a second agrument.',
        params: ['params', 'seed']
    },
    alias: {
        summary: '' +
            'Creates signed createAlias transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed AliasTransaction as a second agrument.',
        params: ['params', 'seed']
    },
    setScript: {
        summary: '' +
            'Creates signed setScript transaction',
        description: '' +
            'You can use this function with multiple seeds. ' +
            'In this case it will sign transaction accordingly ' +
            'and will add one proof per seed. Also you can use ' +
            'already signed SetScriptTransaction as a second agrument.',
        params: ['params', 'seed']
    }
};
/**
 * Variables types vocabulary
 *
 * @static
 * @member {object} types
 */
WavesConsoleAPIHelp.types = {
    tx: {
        summary: 'Transaction object obtained from WavesTransactions library',
        type: 'object'
    },
    code: {
        summary: 'Text of the contract',
        type: 'string'
    },
    seed: {
        optional: true,
        summary: 'Seed string obtained from node (optional, env.SEED by default)',
        type: 'string'
    },
    func: {
        summary: 'Name of the function from API',
        type: 'function'
    },
    params: {
        summary: 'Object with transactions properties',
        type: 'object'
    },
    tabName: {
        summary: 'Tab name for web-ide or vscode plugin',
        type: 'string'
    },
    keyPairOrSeed: {
        summary: 'Seed string or keyPair object from keyPair() function',
        type: 'string'
    },
    apiBase: {
        optional: true,
        summary: 'Url of the node. E.x. "https://nodes.wavesplatform.com". (optional, env.API_BASE by default)',
        type: 'string'
    }
};
exports.WavesConsoleAPIHelp = WavesConsoleAPIHelp;
